;+
FUNCTION xy2lonlat, ang, date0, arcmin=arcmin
; NAME:
;        xy2lonlat
; PURPOSE:
;          Converts arcsec-from-suncenter coordinates to heliographic
;	   coordinates (inverse of latlon2xy).
; METHOD:  
; CALLING SEQUENCE:  
; INPUT:
;	ang	- is a vector of angles from sun center in default units of
;		  arcseconds.  It should be 2xN.
;			(0,*) = angle in E/W direction with W positive
;			(1,*) = angle in N/S direction with N positive
;	date	- Unless the date is supplied, today's date is used.
;		  This will affect the Solar axial tilt, B0. 
; OUTPUT:
;	lonlat  - Heliographic coordinates in decimal degrees.
;                       (0,*) = longitude (degrees) W positive
;                       (1,*) = latitude (degrees) N positive
;		  If the input coordinates are off the solar limb, 
;		  the output is the radial projection back to the nearest 
;		  point on the limb.
;		  Output is DOUBLE)
; OPTIONAL KEYWORD INPUT:
;	arcmin	- when set the input is in arcminutes.
; OPTIONAL KEYWORD OUTPUT:
; CAUTIONS:    - Points off the limb are projected back to the limb.
;              - The x and y are in solar coordinates (solar W and solar N).
;                There is no P-angle here.
; HISTORY:
;     T. Metcalf 2001-Oct-17  This is an exact replacement for conv_a2h which
;                             is only approximate.
;-

nout = n_elements(ang)/2

;-------------------- Get the Date
;
if (n_elements(date0) eq 0) then date = anytim2ints(!stime) $
                        else date = anytim2ints(date0)
if ((n_elements(date) ne nout) and (n_elements(date) ne 1)) then begin
    message, 'Improper number of dates.  Using first date for all points.', /info
    date = date(0)
endif

ans=get_rb0p(date)
if n_elements(ans(0,*)) eq 1 then begin		
  b0=ans(1,0)					; get B0 
  sunr=ans(0,0)
endif else begin
  b0=ans(1,*)					
  sunr=ans(0,*)
endelse

x = double(ang[0,*])
y = double(ang[1,*])

if keyword_set(arcmin) then begin
   x = x * 60.
   y = y * 60.
endif

rho = sqrt(x^2+y^2)<sunr   ; don't let rho off the limb
outrho = rho/sunr

arcsec2rad = 4.84813681d-6
rho = asin(sin(rho*arcsec2rad)/sin(sunr*arcsec2rad)) - rho*arcsec2rad
theta = -atan(-x,y)
srho = sin(rho)

lat = asin(sin(b0)*cos(rho)+cos(b0)*srho*cos(theta))
lon = asin(srho*sin(theta)/cos(lat))

lonlat = double(ang)
lonlat[0,*] = lon*180.0/!dpi
lonlat[1,*] = lat*180.0/!dpi

return, lonlat

end
